app2 <- function(kernel_exp, de_exp, all_exp, de_genes){
  
#INPUTS:
  #kernel_exp: expressions of kernel genes/probes over the specified conditions, a numeric matrix s.t. each row corresponds to the expression vector of a kernel gene
  #de_exp: expressions of differentially expressed genes/probes over the specified conditions, a numeric matrix s.t. each row corresponds to the expression vector of a differntially expressed gene
  #all_exp: expressions of all genes/probs in microarray over the specified conditions (for p-value calculation)
  #de_genes: list of differentially expressed genes, gene in row_i has its expression vector in row_i of 'de_exp'
  
#OUTPUTS: 
  #de_scores: KCS scores of differentially expressed genes
  #p-values of the KSC scores
  
  # cluster kernels
  clusters <- clustering( kernel_exp, 0.8, 0.05 )
  centres  <- update_centres(clusters, kernel_exp)
 
  # remove singleton clusters
  updated_clusters <- rmv_single( clusters, centres)
  updated_centres  <- update_centres(updated_clusters, kernel_exp)
  
  #calculate KCS scores
  scores_de <- kcs(de_exp, updated_centres, kernel_exp )
    
  # randomly sample from all gene'probes expressions for the permutation test
  sample_exp <- all_exp[sample(nrow(all_exp)),]
  scores_sample <- kcs(sample_exp[1:1000,], updated_centres, kernel_exp)
  
  # permutation test, calculate KSC p-values
  de_pvalues <- perm_test(scores_de, scores_sample)
  
  return(list(scores_de,de_pvalues))
}
 
